@@ -110,7 +110,9 @@ arma::mat find_PB_rnd_local_search(arma::mat M, int rep = 1){
 arma::vec get_pc1(arma::mat M){
   arma::vec b_old = arma::zeros<arma::vec>(M.n_cols);
   arma::vec b_new = arma::ones<arma::vec>(M.n_cols);
-  while( max(abs(b_old-b_new)) > 10e-5){
+  int iter = 0;
+  while( max(abs(b_old-b_new)) > 10e-5 & iter < 1000){
+    iter++;
     b_old = b_new;
     b_new = normalise(M * b_old);
   }
@@ -175,9 +177,11 @@ arma::mat find_PB_pc_local_search(arma::mat X){
     PB.push_back(SOLS[iBestSolution]);
     // Rcpp::Rcout << "Best solution found" << std::endl;
     // SOLS[iBestSolution].print_status(true,true,true);
+
     pb_mat.col(l) = balance(K,PB[l]);
     Xclr = Xclr - Xclr * pb_mat.col(l) * pb_mat.col(l).t();
     Mclr = cov(Xclr);
+
     arma::vec PC1 = get_pc1(Mclr);
 
 
@@ -222,6 +226,70 @@ arma::mat find_PB_pc_local_search(arma::mat X){
   return(pb_mat);
 }
 
+// [[Rcpp::export]]
+arma::mat find_PB(arma::mat X){
+  arma::mat M = cov(log(X));
+
+  int K = M.n_cols;
+
+  std::vector<SBP> PB;
+  arma::mat pb_mat = arma::mat(K,K-1);
+  std::vector<SBP> SOLS;
+
+  SOLS.push_back(SBP(M, default_node(K)));
+
+  SOLS.back().optimal();
+  for(int l=0;l<K-1;l++){
+    //Rcpp::Rcout << "Starting step " << l + 1 << " of " << K-1 << std::endl;
+    //print_list(SOLS);
+    double vBestSolution = 0;
+    int iBestSolution = -1;
+    for(unsigned int i =0; i< SOLS.size();i++){
+      double v = SOLS[i].var();
+      if(v > vBestSolution){
+        vBestSolution = v;
+        iBestSolution = i;
+      }
+    }
+    PB.push_back(SOLS[iBestSolution]);
+    // Rcpp::Rcout << "Best solution found" << std::endl;
+    // SOLS[iBestSolution].print_status(true,true,true);
+    pb_mat.col(l) = balance(K,PB[l]);
+
+    int n = SOLS[iBestSolution].get_n();
+    int nL = SOLS[iBestSolution].getL().n_elem;
+    int nR = SOLS[iBestSolution].getR().n_elem;
+    if(n > nL + nR){
+      //Rcpp:: Rcout << "Start top...";
+      SOLS.push_back(SOLS[iBestSolution].top());
+      SOLS.back().optimal();
+      //SOLS.back().print_status(true,true,false);
+      //Rcpp::Rcout << "End top" << std::endl;
+    }
+    if(nL > 1){
+      //Rcpp:: Rcout << "Start left..." << std::endl;
+      SOLS.push_back(SOLS[iBestSolution].left());
+      SOLS.back().optimal();
+      //SOLS.back().print_status(true,true,false);
+      //Rcpp::Rcout << "End left" << std::endl;
+    }
+    if(nR > 1){
+      //Rcpp:: Rcout << "Start right...";
+      SOLS.push_back(SOLS[iBestSolution].right());
+      SOLS.back().optimal();
+      //SOLS.back().print_status(true,true,false);
+      //Rcpp::Rcout << "End right" << std::endl;
+    }
+
+    SOLS[iBestSolution] = SOLS.back();
+    SOLS.pop_back();
+
+    Rcpp::checkUserInterrupt();
+  }
+
+  return(pb_mat);
+}
+
 // [[Rcpp::export]]
 arma::mat find_PB2(arma::mat M, int random = 100, int optim = 0){
   int K = M.n_cols;
@@ -523,7 +591,6 @@ arma::mat find_PB5(arma::mat X){
   return(pb_mat);
 }
 
-
 // Helpers
 std::map<int,arma::uvec> default_node(int size){
   std::map<int,arma::uvec> node;
